<?php

/**
 * @file
 * Alternative page for the search module, for the user search feature,
 * searching by email addresses.
 */

/**
 * Replace the user_search_info() core (search) function
 *
 * Adapt the user search function. The switch between thoses functions is done
 * thanks to the dbee_module_implements_alter() function during the
 * 'search_info' hook.
 */
function dbee_user_search_execute($keys = NULL, $conditions = NULL) {
  $find = array();
  // Replace wildcards with MySQL/PostgreSQL wildcards.
  $origin_keys = $keys;
  $keys = preg_replace('!\*+!', '%', $keys);
  $query = db_select('users')->extend('PagerDefault');
  $query->fields('users', array('uid'));
  if (user_access('administer users')) {
    // Administrators can also search in the otherwise private email field.
    $query->fields('users', array('mail'));
    // HERE we edit the code :
    // Select the custom method.
    if (variable_get('dbee_custom_search', 0)) {
      // The custom method is not the default one. It is faster but less
      // powerfull because it will return the user only if the complet mail
      // string is provided.
      $crypted_key = dbee_convert($keys);
      if ($crypted_key) {
        $query->condition(db_or()->
          condition('name', '%' . db_like($keys) . '%', 'LIKE')->
          condition('mail', $crypted_key));
      }
      else {
        $query->condition('name', '%' . db_like($keys) . '%', 'LIKE');
      }
    }
    else {
      // As powerfull as the core function but consumme a lot more ressources,
      // depending on how much users are stored. The serach will succeed even
      // on partial key. As an example, with this method, seraching
      // 'n@provider.com' will returns (if it exists) 'john@provider.com'.
      // Look for result with the custom search function :
      $mail_uids = dbee_advanced_user_search($origin_keys);
      if (!empty($mail_uids)) {
        $query->condition(db_or()->
          condition('name', '%' . db_like($keys) . '%', 'LIKE')->
          condition('uid', $mail_uids));
      }
      else {
        $query->condition('name', '%' . db_like($keys) . '%', 'LIKE');
      }
    }
  }
  else {
    $query->condition('name', '%' . db_like($keys) . '%', 'LIKE');
  }
  $uids = $query
    ->limit(15)
    ->execute()
    ->fetchCol();
  $accounts = user_load_multiple($uids);

  $results = array();
  foreach ($accounts as $account) {
    $result = array(
      'title' => format_username($account),
      'link' => url('user/' . $account->uid, array('absolute' => TRUE)),
    );
    if (user_access('administer users')) {
      $result['title'] .= ' (' . $account->mail . ')';
    }
    $results[] = $result;
  }

  return $results;
}

/**
 * Search a user via php for %key% feature
 *
 * Search a incomplete mail : thanks to this fonction, the key john@example.com
 * will return smith.john@example.com.
 *
 * @param $keys
 *   A string, the resarch string.
 *
 * @return
 *   An array ah the uid that match the keys.
 */
function dbee_advanced_user_search($keys = NULL) {

  // Prepare the key
  // The database email is already lower case.
  $search_key = drupal_strtolower($keys); // '%n1@example.com%'
  // Delete the first and last '%' if they exists.
  $search_key = preg_replace('/^%|%$/', '', $search_key); // 'n1@example.com'
  // Equivalent pattern to the quotemeta() php function but without escaping
  // '+' and '*'.
  $pattern1 = '/([\.\?\[\^\]\$\(\)])/';
  $pattern = preg_replace($pattern1, '\\\$1', $search_key);
  // In order to act as the database search would do, convert '+', '*' and '%'
  // to '.*'.
  $pattern2 = '/([\+\*\%])/';
  $pattern = '!' . preg_replace($pattern2, '.*', $pattern) . '!';
  // Uncrypt all users.
  // @TODO : Try to use the user_load_multiple() cache if the entry exists, but
  // without loading the whole user when it does not exist, for performance
  // raisons.
  $all_mails = db_query('SELECT uid, mail FROM {users} WHERE uid <> :uid', array(':uid' => 0));
  $result = array();
  while ($account = $all_mails->fetchAssoc()) {
    // Initialisation.
    $mail = $account['mail'];
    $uid = $account['uid'];
    $uncrypted_mail = dbee_decrypt($mail);
    // Search the key
    if (preg_match($pattern, $uncrypted_mail)) {
      $result[] = $uid;
    }
  }
  return $result;
}

/**
 * Implements hook_search_info().
 *
 * To use custom variable $module for the search_data() function.
 */
function dbee_search_view($module = NULL, $keys = '') {
  // HERE edit the core function :
  module_load_include('inc', 'search', 'search.pages');
  $info = FALSE;
  $redirect = FALSE;
  $keys = trim($keys);
  // Also try to pull search keywords out of the $_REQUEST variable to
  // support old GET format of searches for existing links.
  if (!$keys && !empty($_REQUEST['keys'])) {
    $keys = trim($_REQUEST['keys']);
  }

  if (!empty($module)) {
    $active_module_info = search_get_info();
    if (isset($active_module_info[$module])) {
      $info = $active_module_info[$module];
    }
  }

  if (empty($info)) {
    // No path or invalid path: find the default module. Note that if there
    // are no enabled search modules, this function should never be called,
    // since hook_menu() would not have defined any search paths.
    $info = search_get_default_module_info();
    // Redirect from bare /search or an invalid path to the default search path.
    $path = 'search/' . $info['path'];
    if ($keys) {
      $path .= '/' . $keys;
    }
    drupal_goto($path);
  }

  // Default results output is an empty string.
  $results = array('#markup' => '');
  // Process the search form. Note that if there is $_POST data,
  // search_form_submit() will cause a redirect to search/[module path]/[keys],
  // which will get us back to this page callback. In other words, the search
  // form submits with POST but redirects to GET. This way we can keep
  // the search query URL clean as a whistle.
  if (empty($_POST['form_id']) || $_POST['form_id'] != 'search_form') {
    $conditions =  NULL;
    if (isset($info['conditions_callback']) && function_exists($info['conditions_callback'])) {
      // Build an optional array of more search conditions.
      $conditions = call_user_func($info['conditions_callback'], $keys);
    }
    // Only search if there are keywords or non-empty conditions.
    if ($keys || !empty($conditions)) {
      // Log the search keys.
      watchdog('search', 'Searched %type for %keys.', array('%keys' => $keys, '%type' => $info['title']), WATCHDOG_NOTICE, l(t('results'), 'search/' . $info['path'] . '/' . $keys));

      // Collect the search results.
      // HERE we edit the core function :
      if ($info['module'] == 'user') {
        $search_data_prefix = 'dbee_user';
      }
      else {
        $search_data_prefix = $info['module'];
      }
      $results = search_data($keys, $search_data_prefix, $conditions);
    }
  }
  // The form may be altered based on whether the search was run.
  $build['search_form'] = drupal_get_form('search_form', NULL, $keys, $info['module']);
  $build['search_results'] = $results;

  return $build;
}
